<?php

/* Written by Bernd Herd, http://www.herdsoft.com/ 2012.
 * Updated with historic CO2 data from Etheridge et al., 1996 December 2014.
 *
 * This is Free Software licensed under The GNU General Public License
 * Version 2. See attached file COPYING for details.
 */

// Find language to use for strings
if (isset($_SERVER['HTTP_ACCEPT_LANGUAGE']))
{
  $languages = explode( ',', $_SERVER['HTTP_ACCEPT_LANGUAGE'] );
  //$languages = array( $_SERVER['HTTP_ACCEPT_LANGUAGE'] );
  for ($i = 0; $i<count($languages) && !isset($lang); $i++)
    {
      $l = $languages[$i];
      if (strlen($l) < 100 &&
          substr($l, 0, 2)=='de')
        {
          $lang='de';
        }
      else if (strlen($l) < 100 &&
               substr($l, 0, 2)=='en')
        {
          $lang='en';
        }
    }
}

if (!isset($lang)) $lang='en';

// Get parameters from GET request
$total_width =
  isset($_GET['width']) &&
  is_numeric($_GET['width'])
  ? $_GET['width']
  : 640;

$total_height =
  isset($_GET['height']) &&
  is_numeric($_GET['height'])
  ? $_GET['height']
  : 480;

$widget_alias =
  isset($_GET['alias']) &&
  is_numeric($_GET['alias'])
  ? $_GET['alias']
  : 2;

if ($widget_alias < 1) $widget_alias=1;
if ($widget_alias > 4) $widget_alias=4;

$total_width = $total_width * $widget_alias;
$total_height = $total_height * $widget_alias;

$widget_title =
  isset($_GET['title']) && strlen($_GET['title'])<100
  ? $_GET['title']
  : ($lang=='de' ? 'Weltklima-Widget' : 'World climate Widget');

$widget_temp_axis =
  isset($_GET['temp_axis']) && strlen($_GET['temp_axis'])<100
  ? $_GET['temp_axis']
  : ($lang=='de' ? 'Temperaturabweichung (C)' : 'Temperature Anomaly (C)');

$widget_co2_axis =
  isset($_GET['co2_axis']) && strlen($_GET['co2_axis'])<100
  ? $_GET['co2_axis']
  : ($lang=='de' ? 'CO2 Konzentration (ppm)' : 'CO2 Concentration (ppm)');

$widget_temp_legend =
  isset($_GET['temp_legend']) && strlen($_GET['temp_legend'])<100
  ? $_GET['temp_legend']
  : ($lang=='de' ? 'Temperatur' : 'Global Temp');

$widget_co2_legend =
  isset($_GET['co2_legend']) && strlen($_GET['co2_legend'])<100
  ? $_GET['co2_legend']
  : ($lang=='de' ? 'CO2 (ppm)' : 'CO2 (ppm)');

$widget_sun_legend =
  isset($_GET['sun_legend']) && strlen($_GET['sun_legend'])<100
  ? $_GET['sun_legend']
  : ($lang=='de' ? 'Sonne' : 'Sun');

$widget_ice_legend =
  isset($_GET['ice_legend']) && strlen($_GET['ice_legend'])<100
  ? $_GET['ice_legend']
  : ($total_width/$widget_alias >= 600 && $total_height/$widget_alias >= 400)
     ? ($lang=='de' ? 'Eisbohrkern CO2 (ppm)' : 'Ice core CO2 (ppm)')
     : ($lang=='de' ? 'Ice CO2' : 'Ice CO2');

$widget_start_year =
  isset($_GET['start_year']) &&
  is_numeric($_GET['start_year'])
  ? $_GET['start_year']
  : 1950;

$temp_rgb =
  isset($_GET['temp_rgb']) &&
  strlen($_GET['temp_rgb'])==6
  ? $_GET['temp_rgb']
  : "b2b2b2";

$co2_rgb =
  isset($_GET['co2_rgb']) &&
  strlen($_GET['co2_rgb'])==6
  ? $_GET['co2_rgb']
  : "1786ec";

$ice_rgb =
  isset($_GET['ice_rgb']) &&
  strlen($_GET['ice_rgb'])==6
  ? $_GET['ice_rgb']
  : "178600";


/// Class to hold one x/y pair for a graph
class CMeasurement
{
  public $x = 0.0;
  public $y = 0.0;
};


// Constants used later in the program
$co2_factor = 0.008;
$co2_offset = 320.0;

$sun_factor = 0.0006;
$sun_offset = -0.04;

$fontfile = "./FreeSans.ttf";
$ttfsize = 10.0*$widget_alias;


/**
 * Loads temperatures from GLB.Ts+dSST.txt and returns
 * an array of CMeasurement objects representing the content.
 *
 * @return Array of X/Y values
 */
function load_glb()
{
  global $widget_start_year;

  $fd = fopen("GLB.Ts+dSST.txt", "r");
  
  $index = 0;
  while (!feof($fd))
	{
	  $line = fgets($fd);
	  if (sscanf($line, "%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d", $year, 
				 $jan, $feb, $mar, 
				 $apr, $may, $jun,
				 $jul, $aug, $sep,
				 $oct, $nov, $dec,
				 $j_d, $d_n,
				 $djf, $mam, $jja,
				 $son, $year2)==20 &&
		  $year >= $widget_start_year
		  )
		{ 
		  $measurement = new CMeasurement;
		  $measurement->x = $year;
		  $measurement->y = $j_d / 100.0;
		  
		  $glb[$index++] = $measurement;
		}
	}
  
  fclose($fd);

  return $glb;
}


/** Update the min/max values to use by scanning
 *  the measurement array for the biggest/smallest values
 *  @param array     Input measurement array
 *  @param minx      Smallest X-Value
 *  @param maxx      Biggest X-Value
 *  @param miny      Smallest Y-Value
 *  @param maxy      Biggest Y-Value
 */
function search_minmax($array, &$minx, &$maxx, &$miny, &$maxy)
{
  for ($index=0; $index<count($array); $index++)
	{
	  if ($maxx < $array[$index]->x)
		{
		  $maxx = $array[$index]->x;
		}
	  if ($minx > $array[$index]->x)
		{
		  $minx = $array[$index]->x;
		}
	  if ($maxy < $array[$index]->y)
		{
		  $maxy = $array[$index]->y;
		}
	  if ($miny > $array[$index]->y)
		{
		  $miny = $array[$index]->y;
		}
	}
}


/**
 * Loads CO2 concentrations from co2_annmean_mlo.txt and returns
 * an array of CMeasurement objects representing the content.
 *
 * @return Array of X/Y values
 */
function load_co2()
{
  global $co2_factor, $co2_offset;
  global $widget_start_year;

  $fd = fopen("co2_annmean_mlo.txt", "r");
  
  $index = 0;
  while (!feof($fd))
	{
	  $line = fgets($fd);
	  if (sscanf($line, "%d %g %g", 
				 $year, $ppm, $unc)==3 &&
          $year >= $widget_start_year
		  )
		{
		  $measurement = new CMeasurement;
		  $measurement->x = $year;
		  $measurement->y = ($ppm-$co2_offset) * $co2_factor;
		  
		  $co2[$index++] = $measurement;
		}
	}
  
  fclose($fd);

  return $co2;
}


/**
 * Loads Historic CO2 concentrations from ice cores
 * from Etheridge et al., 1996.
 * http://cdiac.ornl.gov/ftp/trends/co2/lawdome.smoothed.yr20
 * and returns an array of CMeasurement objects representing the content.
 *
 * @return Array of X/Y values
 */
function load_icecore_co2()
{
  global $co2_factor, $co2_offset;
  global $widget_start_year;

  $fd = fopen("lawdome.smoothed.yr20", "r");
  
  $index = 0;
  while (!feof($fd))
	{
	  $line = fgets($fd);
	  if (sscanf($line, "%d %g", 
				 $year, $ppm)==2 &&
          $year >= $widget_start_year
		  )
		{
		  $measurement = new CMeasurement;
		  $measurement->x = $year;
		  $measurement->y = ($ppm-$co2_offset) * $co2_factor;
		  
		  $co2[$index++] = $measurement;
		}
	}
  
  fclose($fd);

  return $co2;
}


/**
 * Loads yearly average of sunspot counts from yearssn.dat and returns
 * an array of CMeasurement objects representing the content.
 *
 * @return Array of X/Y values
 */
function load_sun()
{
  global $sun_factor, $sun_offset;
  global $widget_start_year;

  $fd = fopen("yearssn.dat", "r");
  
  $index = 0;
  while (!feof($fd))
	{
	  $line = fgets($fd);
	  if (sscanf($line, "%g %g", 
				 $year, $sunspots)==2 &&
		  $year >= $widget_start_year
		  )
		{
		  $measurement = new CMeasurement;
		  $measurement->x = $year;
		  $measurement->y = $sunspots * $sun_factor + $sun_offset;
		  
		  $sun[$index++] = $measurement;
		}
	}
  
  fclose($fd);

  return $sun;
}


/**
 * Draw a line into a image that is wider than the 1-pixel GD normal
 * pen.
 */
function draw_wide($im, $x1, $y1, $x2, $y2, $col)
{
  global $height, $widget_alias;
  if ($height > 100*$widget_alias)
    {
      imagesetthickness($im, 3*$widget_alias);
    }
  imageline($im, $x1, $y1, $x2, $y2, $col);
  imagesetthickness($im, $widget_alias);
}




/**
 * Draw an array of measurement values into
 * the given image, using the specified scaliong factors
 */
function draw_array($im, $array, $left, $bottom, $scale_x, $scale_y, $minx, $miny, $col)
{
  for ($index=0; $index<count($array); $index++)
	{
	  $x = $left    + ($array[$index]->x-$minx) * $scale_x;
	  $y = $bottom  - ($array[$index]->y-$miny) * $scale_y;
	  
	  if ($index > 0)
		{
		  draw_wide($im, $old_x, $old_y, $x, $y, $col);
		  
		}
	  
	  $old_x = $x;
	  $old_y = $y;
	}
}





// Lad the raw data into arrays
$glb = load_glb();
$co2 = load_co2();
$sun = load_sun();
$ice = load_icecore_co2();

// Find the minimum X/Y values to show in the graph
$minx =  10000000;
$maxx = -10000000;
$miny =  10000000;
$maxy = -10000000;

search_minmax($glb, $minx, $maxx, $miny, $maxy);
search_minmax($co2, $minx, $maxx, $miny, $maxy);
search_minmax($sun, $minx, $maxx, $miny, $maxy);
search_minmax($ice, $minx, $maxx, $miny, $maxy);

// Define the size of the graph within the image
$left = 60*$widget_alias;
$width = $total_width - 130*$widget_alias;

$top = 2*$widget_alias;
$height = $total_height - 20*$widget_alias;

if (isset($widget_title) && strlen($widget_title)>0)
  {
    $top += 28*$widget_alias;
    $height -= 28*$widget_alias;
  }

$scale_x = $width / ($maxx-$minx);
$scale_y = $height / ($maxy-$miny);


// Create the image and define colors to use
//$im     = imagecreate($total_width, $total_height);
if ($widget_alias > 1)
  {
    $im     = imagecreate($total_width, $total_height);
    imagesetthickness($im, $widget_alias);
    $white  = imagecolorallocate($im, 255, 255, 255);
    imagefilledrectangle  ( $im, 0, 0, $total_width, $total_height, $white );
  }
 else
   {
     $im     = imagecreate($total_width, $total_height);
     $white  = imagecolorallocate($im, 255, 255, 255);
     imagecolortransparent($im, $white);
   }
//$im     = imagecreatefrompng('irene.png');


$black  = imagecolorallocate($im, 0, 0, 0);
$gray = imagecolorallocate($im, 178, 178, 178);

if (sscanf($temp_rgb, "%02x%02x%02x", $r, $g, $b)==3)
  {
    $temp_color  = imagecolorallocate($im, $r, $g, $b);
  }
 else
   {
     $temp_color  = imagecolorallocate($im, 178, 178, 178);
   }

if (sscanf($co2_rgb, "%02x%02x%02x", $r, $g, $b)==3)
  {
    $co2_color  = imagecolorallocate($im, $r, $g, $b);
  }
 else
   {
     $co2_color  = imagecolorallocate($im, 23, 134, 236);
   }

$sun_color = imagecolorallocate($im, 255, 208, 28);

if (sscanf($ice_rgb, "%02x%02x%02x", $r, $g, $b)==3)
  {
    $ice_color  = imagecolorallocate($im, $r, $g, $b);
  }
 else
   {
     $ice_color  = imagecolorallocate($im, 23, 134, 236);
   }


$bbox = imagettfbbox( $ttfsize  , 0, $fontfile, $widget_title);
$px = (imagesx($im) - ($bbox[2] - $bbox[0])) / 2;
imagettftext($im, $ttfsize, 0, $px, 13*$widget_alias, $black, $fontfile, $widget_title);

imageline($im, $left, $top, $left+$width, $top, $gray);
imageline($im, $left+$width, $top, $left+$width, $top+$height, $gray);
imageline($im, $left+$width, $top+$height, $left, $top+$height, $gray);
imageline($im, $left, $top+$height, $left, $top, $gray);


// Draw ticks for X axis
for ($year = $widget_start_year+10; $year < $maxx; $year += 10)
  {
	$x = $left + ($year - $minx) * $scale_x;

	//imageline($im, $x, $top, $x, $top + $height, $gray);
	imageline($im, $x, $top, $x, $top + 5, $gray);
	imageline($im, $x, $top+$height-5, $x, $top +$height, $gray);

	$string = sprintf("%d", $year);
    $bbox = imagettfbbox( $ttfsize  , 0, $fontfile, $string);
    $px = - ($bbox[2] - $bbox[0]) / 2;
    imagettftext($im, $ttfsize, 0, $x+$px, $top+$height-$bbox[5], $gray, $fontfile, $string);

    if ($width < 200*$widget_alias)
      {
        $year += 10;
      }
  }

// Draw ticks for Y axis
for ($ytick = -0.2; $ytick < $maxy; $ytick += 0.2)
  {
	$y = $top + $height - ($ytick - $miny) * $scale_y;

	//imageline($im, $left, $y, $left+$width, $y, $gray);
	imageline($im, $left, $y, $left+5, $y, $gray);
	imageline($im, $left+$width-5, $y, $left+$width, $y, $gray);

	$string = sprintf("%.1f", $ytick);
    $bbox = imagettfbbox( $ttfsize  , 0, $fontfile, $string);
    $px = ($bbox[2] - $bbox[0]);
    imagettftext($im, $ttfsize, 0, $left-$px-4*$widget_alias, $y-$bbox[5]/2, $temp_color, $fontfile, $string);

	$string = sprintf("%d", $ytick / $co2_factor + $co2_offset);
    $bbox = imagettfbbox( $ttfsize  , 0, $fontfile, $string);
    $px = - ($bbox[2] - $bbox[0]) / 2;
    imagettftext($im, $ttfsize, 0, $left+$width+4*$widget_alias, $y-$bbox[5]/2, $co2_color, $fontfile, $string);

  }



draw_array($im, $sun, $left, $top+$height, $scale_x, $scale_y, $minx, $miny, $sun_color);
draw_array($im, $glb, $left, $top+$height, $scale_x, $scale_y, $minx, $miny, $temp_color);
draw_array($im, $co2, $left, $top+$height, $scale_x, $scale_y, $minx, $miny, $co2_color);
draw_array($im, $ice, $left, $top+$height, $scale_x, $scale_y, $minx, $miny, $ice_color);

if ($height > 150*$widget_alias)
  {
    $legende_x = $left + 20*$widget_alias;
    $legende_y = $total_height/4-10*$widget_alias;

    draw_wide($im, $legende_x, $legende_y, $legende_x+20*$widget_alias, $legende_y, $temp_color);
    imagettftext($im, $ttfsize, 0, $legende_x+25*$widget_alias, $legende_y+5*$widget_alias, $temp_color, $fontfile, $widget_temp_legend);

    $legende_y += 20*$widget_alias;
    draw_wide($im, $legende_x, $legende_y, $legende_x+20*$widget_alias, $legende_y, $co2_color);
    imagettftext($im, $ttfsize, 0, $legende_x+25*$widget_alias, $legende_y+5*$widget_alias, $co2_color, $fontfile, $widget_co2_legend);

    $legende_y += 20*$widget_alias;
    draw_wide($im, $legende_x, $legende_y, $legende_x+20*$widget_alias, $legende_y, $sun_color);
    imagettftext($im, $ttfsize, 0, $legende_x+25*$widget_alias, $legende_y+5*$widget_alias, $sun_color, $fontfile, $widget_sun_legend);

    $legende_y += 20*$widget_alias;
    draw_wide($im, $legende_x, $legende_y, $legende_x+20*$widget_alias, $legende_y, $ice_color);
    imagettftext($im, $ttfsize, 0, $legende_x+25*$widget_alias, $legende_y+5*$widget_alias, $ice_color, $fontfile, $widget_ice_legend);
  }


$bbox = imagettfbbox  ( $ttfsize  , 90.0, $fontfile  , $widget_temp_axis);
$py = (imagesy($im) - $bbox[3]-$bbox[1])/2;
//                size      angle  x                y    color        fontfile   text
imagettftext($im, $ttfsize, 90.0, 25*$widget_alias, $py, $temp_color, $fontfile, $widget_temp_axis);

$bbox = imagettfbbox  ( $ttfsize  , 90.0, $fontfile  , $widget_co2_axis);
$py = (imagesy($im) - $bbox[3]-$bbox[1])/2;
//                size      angle  x                y    color        fontfile   text
imagettftext($im, $ttfsize, 90.0, $total_width-30*$widget_alias, $py, $co2_color, $fontfile, $widget_co2_axis);


//imagepng($im, 'temp.png');

if ($widget_alias > 1)
  {
    // Work Anti-Alias
    $imr = imagecreatetruecolor($total_width/$widget_alias, $total_height/$widget_alias);
    
    imagecopyresampled($imr,$im,0,0,0,0,imagesx($imr),imagesy($imr),imagesx($im),imagesy($im));
    
    header("Content-type: image/png");
    imagepng($imr);
    //imagepng($imr, "temp.png");
    imagedestroy($imr);
  }
 else
   {
     // No Anti-Aliasing... transparent image
    header("Content-type: image/png");
    imagepng($im);
   }

imagedestroy($im);

?>
